# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Combine tree module: use this module can combine more tree to a consensus tree.

"""
import os
import subprocess
from physpetool.softwares.path import getlocalpath


def docontree(input, output, rule):
    """
Combine tree
    :param input: input files
    :param output: output directory
    """
    # get raxml path
    raxmlpath = getlocalpath()
    # run

    # prepare a dir store result
    if not os.path.exists(output):
        os.mkdir(output)
    consensuseCmd = raxmlpath + "/raxmlHPC-PTHREADS-AVX " + " -J " + rule + " -m GTRCAT -z " + input + " -w " + output + " -n T1"
    subprocess.call(consensuseCmd, shell=True)



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The arguments parse module for check module.

"""

import os
from physpetool.utils.checkdb import check_ehcp, check_hcp, check_srna

APP_DESC = ""


def start_args(input):
    """
Arguments parse
    :param input:  Arguments
    """

    annotation_args = input.add_argument_group("CHECK OPTIONS")
    annotation_args.add_argument('-i', action='store', dest="inputfile",
                                 help="Input a TXT file contain species names (abbreviated names) are same \
                                 with KEGG species abbreviation.")
    annotation_args.add_argument('-o', action='store', dest="outputfile",
                                 default='check', help="A directory contain check result. The default directory \
                                name is check")
    annotation_args.add_argument('--hcp', action='store_true', dest="checkhcp",
                                 default=False, help="Check organisms whether supported by KEGG database.")
    annotation_args.add_argument('--ehcp', action='store_true', dest="checkehcp",
                                 default=False, help="check input organisms prepare for extend autobuild tree module.")
    annotation_args.add_argument('--srna', action='store_true', dest="checksrna",
                                 default=False, help="Check organisms whether supported by SILVA database.")


def starting(args):
    """
Staring run combine
    :param args: arguments
    """
    pwd = os.getcwd()

    out_put = os.path.join(pwd, args.outputfile)
    if args.checkehcp:
        check_ehcp(args.inputfile, out_put)
    if args.checkhcp:
        check_hcp(args.inputfile, out_put)
    if args.checksrna:
        check_srna(args.inputfile, out_put)



# ########################## Copyrights and License ############################
#                                                                              #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                             #
#                                                                              #
# This file is part of PhyspeTree.                                             #
# https://xiaofeiyangyang.github.io/physpetools/                               #
#                                                                              #
# PhySpeTree is free software: you can redistribute it and/or modify it under  #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY#
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.           #
#                                                                              #
# ##############################################################################

"""
Module for annotating tree.
"""

from physpetool.database.dbpath import getlocaldbpath
from physpetool.utils.checkIsNum import is_number
from physpetool.utils.checkinputfile import checkFile, readIputFile
from physpetool.utils.colorconvert import random_color
import os

dbpath = getlocaldbpath()

cite_range = u"""#==============================Color labels with ranges================================#
#                                                                                      #
# 1.This is a generated file by PhySpeTree.                                            #
#                                                                                      #
# 2.Users drop this file to iTOL account display the annotations.                      #
#                                                                                      #
# 3.iTOL V3: http://itol.embl.de/                                                      #
#                                                                                      #
# 4.Please cite:                                                                       #
#              Letunic, I., & Bork, P. (2016). Interactive tree of life (iTOL) v3:     #
#              an online tool for the display and annotation of phylogenetic and       #
#              other trees.Nucleic acids research, gkw290.                             #
#                                                                                      #
#======================================================================================#"""

cite_label = u"""#==============================Color labels without ranges=============================#
#                                                                                      #
# 1.This is a generated file by PhySpeTree.                                            #
#                                                                                      #
# 2.Users drop this file to iTOL account display the annotations.                      #
#                                                                                      #
# 3.iTOL V3: http://itol.embl.de/                                                      #
#                                                                                      #
# 4.Please cite:                                                                       #
#              Letunic, I., & Bork, P. (2016). Interactive tree of life (iTOL) v3:     #
#              an online tool for the display and annotation of phylogenetic and       #
#              other trees.Nucleic acids research, gkw290.                             #
#                                                                                      #
#======================================================================================#"""


def readTaxDb():
    """
    prepare Taxonomy list
    :return: taxonomy list
    """
    orgpath = os.path.join(dbpath, "organism.txt")
    organism_list = []
    with open(orgpath) as f:
        for org in f:
            each_org = org.strip().split('\t')
            organism_list.append([each_org[1], each_org[-1]])
    return organism_list


def readTaxDb_id():
    orgpath = os.path.join(dbpath, "silva.txt")
    organism_list = []
    with open(orgpath) as f:
        for org in f:
            each_org = org.strip().split('\t')
            organism_list.append([each_org[0], each_org[-1]])
    return organism_list


def matchInput(input_organism, taxon):
    """
    Match input file with DB
    :param input_organism: input
    :param taxon:
    :return: a match list and annotation dict use annotation
    """
    if is_number(input_organism[0]):
        organism_list = readTaxDb_id()
    else:
        organism_list = readTaxDb()
    # choice range
    taxon_dict = {'kingdom': 0, 'phylum': 1, 'class': 2, 'order': 3}
    id = taxon_dict.get(taxon)
    match_list = []
    anno = []
    for i in input_organism:
        for j in organism_list:
            if i == j[0]:
                temp_anno = j[1].split(';')[id].strip()
                anno.append(j[1].split(';')[id].strip())
                match_list.append([i, temp_anno])
                break
            else:
                pass
    # check annotation number
    unique_anno = list(set(anno))
    length_anno = len(unique_anno)
    # get random color
    color = random_color(length_anno)
    anno_dict = {}
    # get a annotation dict {'plants':'#77E9CC'}
    for line in range(length_anno):
        anno_dict[unique_anno[line]] = color[line]
    return match_list, anno_dict


def colorRange(input, output, taxon):
    """
    Main function to color range
    :param input: input file names
    :param output: output directory path
    :param taxon: choice annotation by ['kingdom', 'phylum', 'class', 'order']
    :return: no
    """

    if not os.path.exists(output):
        os.makedirs(output)
    # writ file name
    fw_name = "range_color_by_" + taxon + ".txt"
    open_path = os.path.join(output, fw_name)
    fw = open(open_path, 'w')
    # write annotation range head
    fw.write(cite_range)
    fw.write('\n')
    fw.write('TREE_COLORS\nSEPARATOR TAB\nDATA\n')
    # check and get input list
    inputfile = checkFile(input)
    input_list = readIputFile(inputfile)
    # get match list and annotation dict to annotation
    match_list, anno_dict = matchInput(input_list, taxon)
    # write to annotation to file
    for line in match_list:
        color = anno_dict[line[1]]
        write_data = "{0}\trange\t{1}\t{2}\n".format(line[0], color, line[1])
        fw.write(write_data)
    fw.close()
    print("Color range by {0} was complete.".format(taxon))
    print("Color range annotation was save in {0}".format(open_path))


def colorLabel(input, output, taxon):
    """
    Main function to color label
    :param input: input file names
    :param output: output directory path
    :param taxon: choice annotation by ['kingdom', 'phylum', 'class', 'order']
    :return: no
    """

    if not os.path.exists(output):
        os.makedirs(output)
    # writ file name
    fw_name = "labels_color_by_" + taxon + ".txt"
    open_path = os.path.join(output, fw_name)
    fw = open(open_path, 'w')
    # write annotation range head
    fw.write(cite_label)
    fw.write('\n')
    fw.write('TREE_COLORS\nSEPARATOR TAB\nDATA\n')
    # check and get input list
    inputfile = checkFile(input)
    input_list = readIputFile(inputfile)
    # get match list and annotation dict to annotation
    match_list, anno_dict = matchInput(input_list, taxon)
    # write to annotation to file
    for line in match_list:
        color = anno_dict[line[1]]
        write_data = "{0}\tlabel\t{1}\n".format(line[0], color)
        fw.write(write_data)
    fw.close()
    print("Color labels by {0} was complete.".format(taxon))
    print("Color labels annotation was save in {0}".format(open_path))



# ########################## Copyrights and License ############################
#                                                                              #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                             #
#                                                                              #
# This file is part of PhySpeTree.                                             #
# https://xiaofeiyangyang.github.io/physpetools/                               #
#                                                                              #
# PhySpeTree is free software: you can redistribute it and/or modify it under  #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY#
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.           #
#                                                                              #
# ##############################################################################

"""
Change labels from abbreviation names to full names.
"""

from physpetool.database.dbpath import getlocaldbpath
from physpetool.utils.checkIsNum import is_number
from physpetool.utils.checkinputfile import checkFile, readIputFile
import os

dbpath = getlocaldbpath()

cite = u"""#=====================================LABELS===========================================#
#                                                                                      #
# 1.This is a generated file by PhySpeTree.                                            #
#                                                                                      #
# 2.Users drop this file to iTOL account display the annotations.                      #
#                                                                                      #
# 3.iTOL V3: http://itol.embl.de/                                                      #
#                                                                                      #
# 4.Please cite:                                                                       #
#              Letunic, I., & Bork, P. (2016). Interactive tree of life (iTOL) v3:     #
#              an online tool for the display and annotation of phylogenetic and       #
#              other trees.Nucleic acids research, gkw290.                             #
#                                                                                      #
#======================================================================================#"""

def taxlist():
    """
    prepare Taxonomy list
    :return: taxonomy list
    """
    orgpath = os.path.join(dbpath, "organism.txt")
    organism_list = []
    with open(orgpath) as f:
        for org in f:
            each_org = org.strip().split('\t')
            organism_list.append([each_org[1], each_org[2]])
    return organism_list


def taxlistid():
    orgpath = os.path.join(dbpath, "silva.txt")
    organism_list = []
    with open(orgpath) as f:
        for org in f:
            each_org = org.strip().split('\t')
            organism_list.append([each_org[0], each_org[1]])
    return organism_list

def annotatingLabels(input, output):
    """
Change labels
    :param input: input a files contain abb names
    :param output: output directory path
    :return: None
    """

    if not os.path.exists(output):
        os.makedirs(output)
    fw_name = "labels.txt"
    open_path = os.path.join(output, fw_name)
    fw = open(open_path, 'w')
    fw.write(cite)
    fw.write('\n')
    fw.write('LABELS\nSEPARATOR TAB\nDATA\n')
    inputfile = checkFile(input)
    input_list = readIputFile(inputfile)

    if is_number(input_list[0]):
        tax_list = taxlistid()
    else:
        tax_list = taxlist()
    for i in input_list:
        for j in tax_list:
            if i == j[0]:
                fw.write("{0}\t{1}\n".format(j[0], j[1]))
    fw.close()
    print("Change abbreviation names to full names complete")
    print("change labels file was save in {0}".format(open_path))



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Create random color value for color range used.

"""

import colorsys
import random

def convert(value):
    """
    convert 2------>16
    :param value: input int type
    :return: str type(16)
    """
    if value == 10:
        return "A"
    elif value == 11:
        return "B"
    elif value == 12:
        return "C"
    elif value == 13:
        return "D"
    elif value == 14:
        return "E"
    elif value == 15:
        return "F"
    return str(value)


def convert_hex(value):
    if value == "A":
        return str(10)
    elif value == "B":
        return str(11)
    elif value == "C":
        return str(12)
    elif value == "D":
        return str(13)
    elif value == "E":
        return str(14)
    elif value == "F":
        return str(15)
    return str(value)


def rgb2hex(rgb):
    """
    Convert RGB to HEX color
    :param rgb: Rge value example(23,32,44)
    :return: Hex value example #??????
    """
    hex = []
    for i in rgb:
        if i == 0:
            h = str(0) + str(0)
        else:
            h_left = i / 16
            h_right = i % 16
            h = convert(h_left) + convert(h_right)

        hex.append(h)
    hex_combine = "#" + ''.join(hex)
    return hex_combine


# #1722DF
def hex2rgb(hex_value):
    """
    Convert hex to rgp
    :param hex_value: string type example "#1722DF"
    :return: a rgb color tuple example (23,34,223)
    """
    hex = hex_value[1:]
    hex_splite = [convert_hex(x) for x in hex if x]
    hex_splite_rgb = splite_string(hex_splite, 2)
    rgb = [int(line[0]) * 16 + int(line[1]) for line in hex_splite_rgb if line]
    return tuple(rgb)


def splite_string(s, n):
    """ splite strings to sub"""
    return [s[i:i + n] for i in range(len(s)) if i % n == 0]


def rand_hsl():
    """
    Random hsl value
    :return: rgb_hsl value
    """
    h = random.uniform(0.02, 0.31) + random.choice([0, 1 / 3.0, 2 / 3.0])
    l = random.uniform(0.3, 0.8)
    s = random.uniform(0.3, 0.8)

    rgb = colorsys.hls_to_rgb(h, l, s)
    return (int(rgb[0] * 256), int(rgb[1] * 256), int(rgb[2] * 256))


def random_color(num):
    """
    get random color value
    :param num: int
    :return: a list contain Hex color value
    """
    color = []
    for i in range(num):
        color.append(rgb2hex(rand_hsl()))
    return color



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Create highly conserved proteins database index database.

"""

import sqlite3

conn = sqlite3.connect('../database/koindex.db')
conn.text_factory = str
c = conn.cursor()
for row in c.execute('SELECT NAME,EUKARYOTES FROM KOINDEX'):
    print (row)

listko = []
with open('../database/protein_ko.txt') as ko:
    for line in ko:
        str = line.strip().split(',')
        listko.append(str[1])

text = ' TEXT, '.join(listko)


createlist = ["create table if not exists keggproindex", "(id integer primary key autoincrement, NAME TEXT, ", text, " TEXT)"]
createsql = ''.join(createlist)

proindex = sqlite3.connect('../database/proindex.db')
proindex.text_factory = str
cursor = proindex.cursor()
cursor.execute(createsql)

orgs = []
with open('../database/organism.txt') as org:
    for line in org:
        str = line.strip().split('\t')
        orgs.append((str[1],))

cursor.executemany("INSERT INTO keggproindex (NAME)\
    VALUES (?)", orgs)
proindex.commit()
cursor.close()


#cursor.execute('DROP TABLE keggproindex')


# check the name
# cursor.execute("select NAME from keggproindex")
# col_name_list = [tuple[0] for tuple in cursor.description]
# print col_name_list












# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The KEGG KO index database.

"""

import sqlite3
import os

print (os.getcwd())
# name,eukaryotes,prokaryotes
listko = []

with open('../database/protein_ko.txt') as ko:
    i = 1
    for line in ko:
        strs = line.strip().split(',')
        strs.insert(0, i)
        i = i + 1
        listko.append(tuple(strs))
print (listko)
conn = sqlite3.connect('../database/koindex.db')
conn.text_factory = str
c = conn.cursor()
c.execute('''CREATE TABLE IF NOT EXISTS KOINDEX
        (ID INT PRIMARY KEY NOT NULL,
        NAME          TEXT NOT NULL,
        EUKARYOTES     TEXT NOT NULL,
        PROKARYOTES    TEXT NOT NULL);''')
c.executemany("INSERT INTO KOINDEX (ID,NAME,EUKARYOTES,PROKARYOTES)\
              VALUES (?,?,?,?)", listko)
conn.commit()
c.close()



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

import os
import time
from physpetool.phylotree.log import getLogging

"""

Concatenate the muscle alignment files to one file.

"""

logconcat = getLogging('concatenate')


def read_fasta_co(fp):
    """read *.fasta file and parse
    :param fp: a fasta format file
    """
    seq_name, seq_pro = None, []
    for line in fp:
        line = line.rstrip()
        if line.startswith(">"):
            if seq_name: yield (seq_name, ''.join(seq_pro))
            seq_name, seq_pro = line, []
        else:
            seq_pro.append(line.replace(" ", ""))
    if seq_name: yield (seq_name, ''.join(seq_pro))


def cocat(each_muscle_alg):
    """read each files retrun name and sequences append to two list
    :param each_muscle_alg: .fasta file alignment by muscle
    :return: sequence name and sequence
    """
    att_name = []
    each_seq = []
    fr = open(each_muscle_alg)
    with fr as seq:
        for seq_name, seq_pro in read_fasta_co(seq):
            att_name.append(seq_name)
            each_seq.append(seq_pro)
    return (att_name, each_seq)


def cocat_path(muscle_alg):
    """
    Function can be invoked do concatenate:
    :param muscle_alg: a path contain all .fasta files are already alignment by muscle
    :return: the path of .phy file location

    """
    result_dir = os.path.dirname(muscle_alg)
    result = 'concatenate.fasta'
    timeformat = '%Y%m%d%H%M%S'
    timeinfo = str(time.strftime(timeformat))
    subdir = 'concatenate' + timeinfo
    concat_dir = os.path.join(result_dir, subdir)
    # crate a directory to store .phy file
    if not os.path.exists(concat_dir):
        os.makedirs(concat_dir)
    # write concatenate result to file
    result_path = os.path.join(concat_dir, result)
    fw = open(result_path, 'w')
    alg_files = os.listdir(muscle_alg)
    dict = {}
    dict_abb = {}
    for eachpro in alg_files:
        path = os.path.join(muscle_alg, eachpro)
        each_k, each_v = cocat(path)
        dict_abb[eachpro] = each_k
        dict[eachpro] = each_v
    # Get all abb names
    all_abb = dict_abb.get(alg_files[0])
    # Concatenate
    for i in all_abb:
        concat = []
        for j in alg_files:
            index_abb = dict_abb.get(j).index(i)
            concat.append(dict.get(j)[index_abb])
        fw.write(i + '\n')
        fw.write(''.join(concat) + '\n')

    logconcat.info("Concatenate highly conserved proteins was completed")
    return result_path



"""Print style"""

STYLE = {
    'fore': {
        'black': 30, 'red': 31, 'green': 32, 'yellow': 33,
        'blue': 34, 'purple': 35, 'cyan': 36, 'white': 37,
    },
    'back': {
        'black': 40, 'red': 41, 'green': 42, 'yellow': 43,
        'blue': 44, 'purple': 45, 'cyan': 46, 'white': 47,
    },
    'mode': {
        'bold': 1, 'underline': 4, 'blink': 5, 'invert': 7,
    },
    'default': {
        'end': 0,
    }
}


def print_style(string, mode='', fore='', back=''):
    """choice the style"""
    mode = '%s' % STYLE['mode'][mode] if mode in STYLE['mode'] else ''
    fore = '%s' % STYLE['fore'][fore] if fore in STYLE['fore'] else ''
    back = '%s' % STYLE['back'][back] if back in STYLE['back'] else ''
    style = ';'.join([s for s in [mode, fore, back] if s])
    style = '\033[%sm' % style if style else ''
    end = '\033[%sm' % STYLE['default']['end'] if style else ''
    return '%s%s%s' % (style, string, end)
print_style('INFO: ', fore='green')


import os

os.chdir("/home/yangfang/PhySpeTree/updateSILVA/")
w_file = open('kegg_abb_to_ncbi_tax.txt', 'a')
with open('genome.txt') as f:
    for line in f:
        tem = line.strip().split(';')[0]
        tem2 = tem.split('\t')[1]
        tem3 = tem2.split(',')
        print(tem3)
        if len(tem3) == 2:
            w_file.write('{0}\t{1}\n'.format(tem3[0].strip(), tem3[1].strip()))
        else:
            w_file.write('{0}\t{1}\n'.format(tem3[0].strip(), tem3[2].strip()))
w_file.close()

# ~~~~~~~~~~~~~~~~~~~~~~~~~~get all SILVA support orgnism~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
import os

file_path = '/home/yangfang/PhySpeTree/updateSILVA/database16s/'
all_pre_file = list(filter(lambda f: not f.startswith('.'), os.listdir(file_path)))
f_w = open('/home/yangfang/PhySpeTree/updateSILVA/support_srna_organism.txt', 'a')
for i in all_pre_file:
    f_w.write("{0}\n".format(i.strip().split('.')[0]))
f_w.close()

# ~~~~~~~~~~~~~~~~~~~~~~~~~~deal with NCBI tax to latin name~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

f_write = open('/home/yangfang/PhySpeTree/updateSILVA/id_to_latin_name.txt', 'a')
tem_id = []
with open('/home/yangfang/PhySpeTree/updateSILVA/taxmap_embl_ssu_ref_nr99_132.txt') as f:
    for line in f:
        tem = line.strip().split('\t')
        silva_id = tem[0] + '.' + tem[1] + '.' + tem[2]
        if tem[5] not in tem_id:
            f_write.write("{0}\t{1}\n".format(tem[5],tem[4]))
            tem_id.append(tem[5])
f_write.close()

# ~~~~~~~~~~~~~~~~~~~map silva id to ncbi id~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
f_write = open('/home/yangfang/PhySpeTree/updateSILVA/map_silva_to_ncbi.txt', 'a')

with open('/home/yangfang/PhySpeTree/updateSILVA/taxmap_embl_ssu_ref_nr99_132.txt') as f:
    for line in f:
        tem = line.strip().split('\t')
        silva_id = tem[0] + '.' + tem[1] + '.' + tem[2]
        f_write.write("{0}\t{1}\n".format(silva_id, tem[5]))

f_write.close()

# ~~~~~~~~~~~~~~~~~~~map silva id to ncbi id~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
f_write = open('/home/yangfang/PhySpeTree/updateSILVA/silva.txt', 'a')
tem_id = []
with open('/home/yangfang/PhySpeTree/updateSILVA/taxmap_embl_ssu_ref_nr99_132.txt') as f:
    for line in f:
        tem = line.strip().split('\t')
        silva_id = tem[0] + '.' + tem[1] + '.' + tem[2]
        if tem[5] not in tem_id:
            f_write.write("{0}\t{1}\t{2}\t{3}\n".format(silva_id, tem[5], tem[4], tem[3]))
            tem_id.append(tem[5])
f_write.close()

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
import os

file_path = '/home/yangfang/PhySpeTree/updateSILVA/database16s/'
all_pre_file = list(filter(lambda f: not f.startswith('.'), os.listdir(file_path)))
for i in all_pre_file:
    new_name = i.strip()
    os.rename('/home/yangfang/PhySpeTree/updateSILVA/database16s/' + i,
              '/home/yangfang/PhySpeTree/updateSILVA/database16s/' + new_name)



f_write = open('/home/yangfang/PhySpeTree/updateSILVA/silva2.txt', 'a')
with open('/home/yangfang/PhySpeTree/updateSILVA/silva.txt') as f:
    for line in f:
        tem = line.strip().split('\t')
        f_write.write("{0}\t{1}\t{2}\n".format(tem[1], tem[2], tem[3]))

f_write.close()



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The main module as enter point and contain a main() function to invoke other
modules as pipeline.
"""

from physpetool.physpe import autobuild
from physpetool.physpe import check
from physpetool.physpe import combine
from physpetool.physpe import build
from physpetool.physpe import iview
from physpetool.version import version_infor
from physpetool.version import version
import argparse
import sys

# PhySpeTree version
physeptree_version = version

APP_DESC = (
    """
          -------------------------------------------------------------------------------------
                      PhySpeTree (%s) - Reconstruct Phylogenetic species Tree

          Citation:
                  PhySpeTree: automatically reconstructing phylogenetic species tree (submitted)

          --------------------------------------------------------------------------------------
          """ % (physeptree_version))


def main():
    start(sys.argv)


def start(input):
    if len(input) > 1:
        command = input[1]
        if command == "version":
            version_infor()
            return

    parser = argparse.ArgumentParser(description=APP_DESC,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    subparser = parser.add_subparsers(title="RCONSTRUCT PHYLOGENETIC TREE")

    # autobuild
    physpe_autobuid = subparser.add_parser("autobuild", help="Auto reconstruct phylogenetic tree")
    physpe_autobuid.set_defaults(func=autobuild.starting)
    autobuild.start_args(physpe_autobuid)

    # combine
    physpe_combine = subparser.add_parser("combine", help="Combine phylogenetic tree")
    physpe_combine.set_defaults(func=combine.starting)
    combine.start_args(physpe_combine)

    # iview
    physpe_iview = subparser.add_parser("iview", help="View tree by iTol")
    physpe_iview.set_defaults(func=iview.starting)
    iview.start_args(physpe_iview)

    # build
    physpe_build = subparser.add_parser("build", help="Extend phylogenetic tree with new species")
    physpe_build.set_defaults(func=build.starting)
    build.start_args(physpe_build)

    # check
    physpe_check = subparser.add_parser("check", help="Check organism database and prepare for extend tree files")
    physpe_check.set_defaults(func=check.starting)
    check.start_args(physpe_check)

    if len(input) == 1:
        print(APP_DESC)
        parser.print_usage()
        print('Please use program option for details')
        print('Example: \n$ PhySpeTree autobuild -h')
        return
    args = parser.parse_args(input[1:])
    # call subparser
    args.func(args)


if __name__ == '__main__':
    main()
    # start([......])



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The arguments parse module for iview module.

"""

import os
from physpetool.view.annotatingtree import colorRange, colorLabel
from physpetool.view.changelabels import annotatingLabels

APP_DESC = ""


def start_args(input):
    """
Arguments parse
    :param input:  Arguments
    """
    taxonomy = ['kingdom', 'phylum', 'class', 'order']
    taxon = ', '.join(taxonomy)
    annotation_args = input.add_argument_group("ANNOTATION OPTIONS")
    annotation_args.add_argument('-i', action='store', dest="inputfile",
                                 help="Input a TXT file contain species names (abbreviated names) are same \
                                with KEGG species abbreviation.")

    annotation_args.add_argument('-o', action='store', dest="outputfile",
                                 default='iview', help="A directory contain the generate configure files. \
                               The directory name is iview")

    annotation_args.add_argument('-r', '--range', action='store_true', dest="colorrange", default=False,
                                 help="Annotating labels with ranges by %s. The default is phylum." % (taxon))

    annotation_args.add_argument('-c', '--color', action='store_true', dest="colorlabel", default=False,
                                 help="Annotating labels without ranges by %s. The default is phylum." % (taxon))

    annotation_args.add_argument('-a', action='store', dest="assign", choices=taxonomy, default='phylum',
                                 help="Colored ranges by user choice form [%s]." % (taxon))

    annotation_args.add_argument('-l', '--labels', action='store_true', dest="labels", default=False,
                                 help="Change species labels from abbreviated names to full names.")


def starting(args):
    """
Staring run combine
    :param args: arguments
    """
    pwd = os.getcwd()
    out_put = os.path.join(pwd, args.outputfile)

    if os.path.isfile(args.inputfile):
        args_input = args.inputfile
    else:
        args_input = os.path.join(pwd, args.inputfile)
    if args.colorrange:
        colorRange(args_input, out_put, args.assign)
    elif args.labels:
        annotatingLabels(args_input, args.outputfile)
    elif args.colorlabel:
        colorLabel(args_input, out_put, args.assign)


# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The log configuration
"""

import logging
import os

from physpetool.utils.printstyle import print_style


def setlogdir(logdir):
    '''set the log directory'''
    # set log color
    logging.addLevelName(logging.INFO, print_style('%s', fore='green') % logging.getLevelName(logging.INFO))
    logging.addLevelName(logging.WARNING, print_style('%s', fore='red') % logging.getLevelName(logging.WARNING))
    ldir = os.path.dirname(logdir)
    writelog = os.path.join(ldir, 'log.log')
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s %(name)s[line:%(lineno)d] %(levelname)s %(message)s',
                        datefmt='%a, %d %b %Y %H:%M:%S',
                        filename=writelog,
                        filemode='w')

    console = logging.StreamHandler()

    console.setLevel(logging.INFO)

    formatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s: %(message)s')

    console.setFormatter(formatter)

    logging.getLogger('').addHandler(console)


def getLogging(name):
    return logging.getLogger(name)



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
The module retrieve and download SSU rRNA sequence from SILVA Database
"""

import ftplib
import os
import time
from physpetool.phylotree.log import getLogging

log_retrieve = getLogging('SSU rRNA DB')


def retrieve16srna(spenamelist, outpath):
    """
retrieve 16s rna form bioinfor.scu.edu.cn
    :param spenamelist: a list contain species names
    :param outpath: output data path
    :return: download file path
    """
    # pares and cheak species names lsit
    # spelist = []
    # for line in spenamelist:
    #     st = line.strip()
    #     spelist.append(st)
    spelist = spenamelist
    log_retrieve.info('Read organisms names success')
    # makdir tmep directory
    timeformat = '%Y%m%d%H%M%S'
    timeinfo = str(time.strftime(timeformat))
    subdir = 'temp/rna_sequence' + timeinfo
    dirname = os.path.dirname(outpath)
    dirname = os.path.join(dirname, subdir)
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    # connect database
    connect = ftplib.FTP("173.255.208.244")
    connect.login('anonymous')
    connect.cwd('/pub/database16s')
    # connect.dir()
    for abb in spelist:
        retrievename = abb + '.fasta'
        downloadfilname = 'rna_sequence' + '.fasta'
        downloadfilnamepath = os.path.join(dirname, downloadfilname)
        fw = open(downloadfilnamepath, 'ab')
        remoteFileName = 'RETR ' + os.path.basename(retrievename)
        connect.retrbinary(remoteFileName, fw.write)
        fw.write(b'\n')
        fw.close()
        log_retrieve.info("Retrieve and download of organism '{0}' SSU rRNA sequence was successful".format(abb))
    connect.quit()

    return dirname



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Module to call clustalw2 to do alignment

"""

# clustalw2 -INFILE=p1.fasta -TYPE=PROTEIN -OUTPUT=FASTA -ALIGN -OUTFILE=test/clustalw_p2.fasta
import os
import subprocess
from physpetool.phylotree.log import getLogging
from physpetool.softwares.path import getlocalpath
from physpetool.utils.outdirforamt import timeformat

logdoclustalw = getLogging('clutalw2')


def doclustalw(indata, outdata, clustalwpara):
    """
    Call clustalw software to do align
    :param indata: a director contain a fasta format file or a fasta format file
    :param outdata: the out is abs path with a file name
    :return: outdata path
    """
    logdoclustalw.debug("clustalw input data:{0}".format(indata))
    type = "-TYPE=DNA"
    if clustalwpara is None:
        clustalwparas = type
    else:
        clustalwparas = type + " " + clustalwpara.lstrip()
    clu_path = getlocalpath()
    out_path = os.path.dirname(outdata)
    doclu_subdir = str(timeformat('temp/rna_sequence'))
    clustalw_dir = os.path.join(out_path, doclu_subdir)
    # check indata type is dir or files?
    if os.path.isdir(indata):
        pro_name = os.listdir(indata)
        if not os.path.exists(clustalw_dir):
            os.makedirs(clustalw_dir)
        out_file = "-OUTFILE=" + os.path.join(clustalw_dir, pro_name[0])
        infile = os.path.join(indata,pro_name[0])
        cmd = clu_path + "/clustalw2 " + "-INFILE=" + infile + " -OUTPUT=FASTA -ALIGN " + out_file + " " + clustalwparas
        subprocess.call(cmd, shell=True)
        logdoclustalw.info("Multiple sequence alignment  by Clustalw2 was completed.")
        out_alg = os.path.join(clustalw_dir, pro_name[0])
        return out_alg
    # indata is a file
    elif os.path.isfile(indata):
        pro_name = indata
        if not os.path.exists(clustalw_dir):
            os.makedirs(clustalw_dir)
        out_file = "-OUTFILE=" + os.path.join(clustalw_dir, pro_name)
        cmd = clu_path + "/clustalw2 " + "-INFILE=" + pro_name + " -OUTPUT=FASTA -ALIGN " + out_file + " " + clustalwparas
        subprocess.call(cmd, shell=True)
        logdoclustalw.info("Multiple sequence alignment  by Clustalw2 was completed.")
        out_alg = os.path.join(clustalw_dir, pro_name)
        return out_alg


def doclustalw_file(indata_files, outdata, clustalwpara):
    """
Call clustalw software to do align
    :param indata_files: a directory contain more than one file
    :param outdata: out file after alignment
    :return: path
    """
    type = "-TYPE=PROTEIN"
    if clustalwpara is None:
        clustalwparas = type
    else:
        clustalwparas = type + " " + clustalwpara.lstrip()

    clu_path = getlocalpath()
    out_path = os.path.dirname(outdata)
    doclu_subdir = str(timeformat('temp/hcp_alignment'))
    clustalw_dir = os.path.join(out_path, doclu_subdir)
    pro_name = os.listdir(indata_files)
    if not os.path.exists(clustalw_dir):
        os.makedirs(clustalw_dir)
    for i in pro_name:
        each_pro = os.path.join(indata_files, i)
        out_file = "-OUTFILE=" + os.path.join(clustalw_dir, i)
        cmd = clu_path + "/clustalw2 " + "-INFILE=" + each_pro + " -OUTPUT=FASTA -ALIGN " + out_file + " " + clustalwparas
        subprocess.call(cmd, shell=True)
    logdoclustalw.info("Multiple sequence alignment by Clustalw2 was completed.")
    return clustalw_dir


if __name__ == '__main__':
    # doclustalw_file("/home/yangfang/physpetools_data/test_clustalw2/input",
    #                 "/home/yangfang/physpetools_data/test_clustalw2/test", "")
    doclustalw("/home/yangfang/physpetools_data/test_clustalw2/16srandata.fasta",
               "/home/yangfang/physpetools_data/test_clustalw2/test", '')



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################
"""
module return the time format dir

"""
import time


class timeformat:
    """The file add time format"""
    subdir = ''

    def __init__(self, sub):
        self.subdir = sub

    def __str__(self):
        time_format = '%Y%m%d%H%M%S'
        timeinfo = str(time.strftime(time_format))
        subdir = self.subdir + timeinfo
        return subdir




# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Create 16s SSU rRNA database.

"""



import os

def read_fasta_parse(fp):
    """read *.fasta file and parse
    :param fp: a fasta format file
    """
    seq_name, seq_pro = None, []
    for line in fp:
        line = line.rstrip()
        if line.startswith(">"):
            if seq_name: yield (seq_name, ''.join(seq_pro))
            seq_name, seq_pro = line, []
        else:
            seq_pro.append(line.replace(" ", ""))
    if seq_name: yield (seq_name, ''.join(seq_pro))


os.chdir('/home/yangfang/PhySpeTree/updateSILVA/')
seq_name = []
seq_pro = []


def parsefastafile(parsefilepath, idfilepath, wdir):
    if not os.path.exists(wdir):
        os.makedirs(wdir)
    ncbi_acc = read_ncbi_id(idfilepath)
    with open(parsefilepath) as read:
        for seq_name, seq_pro in read_fasta_parse(read):
            acc = seq_name.strip().split(' ')[0].replace('>', '')
            ncbi_acc_re = []
            for num in ncbi_acc:
                if num[1] == acc:
                    faname = num[0] + '.fasta'
                    fapath = os.path.join(wdir, faname)
                    fw = open(fapath, 'w')
                    fw.write('>' + num[0] + '\n')
                    fw.write(seq_pro)
                    fw.close()
                    ncbi_acc_re.append(num)
                else:
                    pass
            for line in ncbi_acc_re:
                ncbi_acc.remove(line)

def read_ncbi_id(idfilepath):
    ncbi_acc = []
    ncbi_tem = []
    with open(idfilepath) as readid:
        for line in readid:
            lin = line.strip().split('\t')
            if lin[1] not in ncbi_tem:
                ncbi_tem.append(lin[1].strip())
                ncbi_acc.append([lin[1].strip(), lin[0]])
    return ncbi_acc


parsefastafile('SILVA_132_SSURef_Nr99_tax_silva_trunc.fasta','map_silva_to_ncbi.txt','ftpfiles')


# parsefastafile('silva_test.fasta', 'ncbi_test2.txt', 'testreslult2')


# ncbi_acc = read_ncbi_id('ncbi_to_silva_id.txt')
# print ncbi_acc




# with open('test.fasta') as read:
#     for seq_name, seq_pro in read_fasta_parse(read):
#         acc = seq_name.strip().split(' ')[0].replace('>', '')
#         print acc



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################
"""
function to call RAxML construct tree
"""

import os
import subprocess
from physpetool.phylotree.log import getLogging
from physpetool.softwares.path import getlocalpath

logdoraxml = getLogging('RAxML')


def doraxml(inputfile, outputfile, raxmlpara, thread):
    """
call RAxML method to construct species tree
    :param inputfile: abs path of .phy format files
    :param outputfile: a file contain RAxML result
    """
    raxmlparas = raxmlpara.lstrip()

    raxmlparalist = raxmlparas.split(" ")
    tpara = '-T'
    if tpara in raxmlparalist:
        index = raxmlparalist.index(tpara)
        raxmlparalist.remove(raxmlparalist[index])
        raxmlparalist.remove(raxmlparalist[index])
        raxmlpararet = ' '.join(raxmlparalist)
    else:
        raxmlpararet = raxmlpara
    threadtostr = str(thread)
    raxmlpath = getlocalpath()
    if not os.path.exists(outputfile):
        os.mkdir(outputfile)
    strs = raxmlpath + "/raxmlHPC-PTHREADS-AVX " + "-T " + threadtostr + " " + raxmlpararet
    # cmd command
    cmd = strs + " -s " + inputfile + " -w " + outputfile
    subprocess.call(cmd, shell=True)
    logdoraxml.info("Phylogenetic species tree reconstructed by RAxML was completed")



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""

The module retrieve highly conserved proteins and download from KEGG database


"""

import os
import sqlite3
import time
from physpetool.database.dbpath import getlocaldbpath
from physpetool.phylotree.log import getLogging
from physpetool.tools.keggapi import getprotein

logretrieveprotein = getLogging('KEGG INDEX DB')
KEGGDB = "KEGG_DB_2.0.db"

def getspecies(name, colname):
    """
    Get species protein index for DB
    :param name: a list contain abbreviation species nam
    :param colname: a list contain colname of DB
    :return: a list contain protein index can be retrieved and a match ko list (is a ko id list)
    """
    dbpath = getlocaldbpath()
    db = os.path.join(dbpath, KEGGDB)
    relist = []
    match_ko_name = []
    conn = sqlite3.connect(db)
    conn.text_factory = str
    c = conn.cursor()
    connect = "' OR NAME = '".join(name)
    for ko in colname:

        query = "SELECT " + ko + " FROM proindex WHERE NAME = '" + connect + "'"
        c.execute(query)
        ids = list(c.fetchall())
        idslist = [str(x[0]) for x in ids]
        if 'None' not in idslist:
            relist.append(idslist)
            match_ko_name.append(ko)
        else:
            pass
    c.close()
    return relist, match_ko_name


def getcolname():
    """get BD colnames"""
    dbpath = getlocaldbpath()
    db = os.path.join(dbpath, KEGGDB)
    conn = sqlite3.connect(db)
    conn.text_factory = str
    c = conn.cursor()
    c.execute("SELECT * FROM proindex")
    col_name_list = [tuple[0] for tuple in c.description]
    c.close()
    return col_name_list[2:]


def splist(l, s):
    """split a list to sub list contain s"""
    return [l[i:i + s] for i in range(len(l)) if i % s == 0]


def retrieveprotein(proindexlist, outpath, matchlist):
    """
    Retrieve proteins form Kegg DB
    :param proindexlist: a list contain protein index
    :param outpath: user input outpath
    :return: retrieve protein path
    """
    timeformat = '%Y%m%d%H%M%S'
    timeinfo = str(time.strftime(timeformat))
    subdir = 'temp/conserved_protein' + timeinfo
    dirname = os.path.dirname(outpath)
    dirname = os.path.join(dirname, subdir)
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    p = 1
    for index in proindexlist:
        hcp_pro_name = hcp_name(matchlist[p - 1])

        wfiles = "{0}/p{1}.fasta".format(dirname, p)
        f = open(wfiles, "a")
        spprolist = splist(index, 10)
        for id in spprolist:
            query = "+".join(id)
            protein = getprotein(query)
            protein = protein.readlines()
            protein = [x.decode('utf-8') for x in protein]
            for i in protein:
                if i.startswith(">"):
                    abbname = i.split(":")[0] + "\n"
                    f.write(abbname)
                else:
                    f.write(i)
        f.close()
        logretrieveprotein.info(
            "Retrieve and download of highly conserved protein '{0}' was successful store in p{1}.fasta file".format(hcp_pro_name, str(p)))
        p += 1
    logretrieveprotein.info("Retrieve from KEGG database " + str(p - 1) + " highly conserved proteins")
    return dirname


def doretrieve(specieslistfile, outpath):
    '''main to  retrieve protein from kegg db'''
    # spelist = []
    # for line in specieslistfile:
    #     st = line.strip()
    #     spelist.append(st)
    spelist = specieslistfile
    logretrieveprotein.info("Read organisms names success")
    colname = getcolname()
    relist, matchlist = getspecies(spelist, colname)
    dirpath = retrieveprotein(relist, outpath, matchlist)
    return dirpath


def hcp_name(index):
    """get highly conserved protein names from ko list"""
    ko_path = getlocaldbpath()
    pro_ko = os.path.join(ko_path, "protein_ko.txt")
    with open(pro_ko) as ko:
        for line in ko:
            name = line.strip().split(',')
            if name[1] == index:
                return name[0]


if __name__ == '__main__':
    print (getcolname())
    print (getspecies(['zma'], ['K02925']))



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Connect 16s SSU rRNA database.

"""


import ftplib
import os

connect = ftplib.FTP("173.255.208.244")
connect.login('anonymous')
connect.cwd('/pub/database16s/')
os.chdir('/home/yangfang/')
downloadfilname = "999421.fasta"
f = open(downloadfilname, 'wb')
ftpfilename = "999421.fasta"
remoteFileName = 'RETR ' + os.path.basename(ftpfilename)
connect.retrbinary(remoteFileName, f.write, 1024)
f.close()


connect.quit()



# ########################## Copyrights and License #############################
#                                                                               #
# Copyright 2016 Yang Fang <yangfangscu@gmail.com>                              #
#                                                                               #
# This file is part of PhySpeTree.                                              #
# https://xiaofeiyangyang.github.io/physpetools/                                #
#                                                                               #
# PhySpeTree is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)     #
# any later version.                                                            #
#                                                                               #
# PhySpeTree is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS     #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more  #
# details.                                                                      #
#                                                                               #
# You should have received a copy of the GNU Lesser General Public License      #
# along with PhySpeTree. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                               #
# ###############################################################################

"""
Module to call muscle to do alignment
"""

import subprocess
import os
import os.path
import time
from physpetool.phylotree.log import getLogging
from physpetool.softwares.path import getlocalpath

logdomuscle = getLogging('muscle')


# muscle -in process_L1.txt -out process_L1.afa -maxiters 100
def domuscle(indata, outdata, musclepara):
    """
    call muscle software to do align
    :param indata: a director contain a fasta format file or a fasta format file
    :param outdata: the out is abs path with a file name
    :return: outdata path
    """
    muscleparas = musclepara.lstrip()
    mupath = getlocalpath()
    out_path = os.path.dirname(outdata)
    timeformat = '%Y%m%d%H%M%S'
    timeinfo = str(time.strftime(timeformat))
    subdir = 'temp/rna_alignment' + timeinfo
    muscle_dir = os.path.join(out_path, subdir)
    # check indata type
    if os.path.isdir(indata):
        pro_name = os.listdir(indata)
        if not os.path.exists(muscle_dir):
            os.makedirs(muscle_dir)
        out_alg = os.path.join(muscle_dir, pro_name[0])
        each_pro = os.path.join(indata, pro_name[0])
        cmd = mupath + "/muscle -in " + each_pro + " -out " + out_alg + " " + muscleparas
        subprocess.call(cmd, shell=True)
        logdomuscle.debug('muscle result path:{0}'.format(out_alg))
        logdomuscle.info("Multiple sequence alignment by Muscle was completed.")
        return out_alg
    elif os.path.isfile(indata):
        pro_name = indata
        if not os.path.exists(muscle_dir):
            os.makedirs(muscle_dir)
        out_alg = os.path.join(muscle_dir, pro_name)
        each_pro = pro_name
        cmd = mupath + "/muscle -in " + each_pro + " -out " + out_alg + " " + muscleparas
        subprocess.call(cmd, shell=True)
        logdomuscle.debug('muscle result path:{0}'.format(out_alg))
        logdomuscle.info("Multiple sequence alignment by Muscle was completed.")
        return out_alg


def domuscle_file(indata_files, outdata, musclepara):
    """
call muscle software to do align
    :param indata_files: a directory contain more than one file
    :param outdata: out file after alignment
    :return: path
    """
    muscleparas = musclepara.lstrip()
    mupath = getlocalpath()
    out_path = os.path.dirname(outdata)
    timeformat = '%Y%m%d%H%M%S'
    timeinfo = str(time.strftime(timeformat))
    subdir = 'temp/alignment' + timeinfo
    muscle_dir = os.path.join(out_path, subdir)
    # muscle_dir = os.path.join(indata_files, 'muscle_alignment')
    pro_name = os.listdir(indata_files)
    if not os.path.exists(muscle_dir):
        os.makedirs(muscle_dir)
    for i in pro_name:
        out_alg = os.path.join(muscle_dir, i.split('.')[0])
        each_pro = os.path.join(indata_files, i)
        cmd = mupath + "/muscle -in " + each_pro + " -out " + out_alg + " " + muscleparas
        subprocess.call(cmd, shell=True)
    logdomuscle.info("Multiple sequence alignment by Muscle was completed.")
    return muscle_dir
